#include <gtk/gtk.h>
#include <glib.h>
#include <popt.h>
#include <stdio.h>
#include <libintl.h>
#include <stdlib.h>

#include "defs.h"
#include "file.h"
#include "config.h"

void parse_buttons(GString* buttons, OPTIONS *my_config) {
	char *begin, *end;
	char separator;
	int i;
	gint value;
	gchar *tmp;
	
	tmp = (gchar*)malloc(sizeof(char)*MAX_STRING_LENGTH);
	
	begin = buttons->str;
	if(begin[0]) {
		my_config->value = g_array_new(TRUE, TRUE, sizeof(gint));
		my_config->label = g_ptr_array_new();
		my_config->number = g_ptr_array_new();
	}
	
	end = begin;
	i = 0;
	while(end[0] != 0) {
		while((end[0] != 0) && (end[0] != ':') && (end[0] != ',')) end++;
		separator = end[0];
		end[0] = 0;
		g_ptr_array_add(my_config->label, g_string_new(begin));
		snprintf(tmp, MAX_STRING_LENGTH, "%d", i);
		g_ptr_array_add(my_config->number, g_string_new(tmp));
		end[0] = separator;
		
		if(separator) {
			begin = end+1;
			end = begin;
			if(separator == ':') {
				while((end[0] != 0) && (end[0] != ':') && (end[0] != ',')) end++;
				separator = end[0];
				end[0] = 0;
				value = atoi(begin);
				g_array_append_val(my_config->value, value);
				end[0] = separator;
				if(separator) {
					begin = end+1;
					end = begin;
				}
			} else {
				value = 0;
				g_array_append_val(my_config->value, value);
			}
		}
		i++;
	}
	free(tmp);
}

void usage(poptContext optCon, int exitcode, char *error) {
	poptPrintUsage(optCon, stderr, 0);
	if (error) fprintf(stderr, "%s", error);
	exit(exitcode);
}

OPTIONS *parse_command_line (int argc, const char **argv) {
	char* buttons;
	char* default_label;
	char* filename;
	char* geometry;
	const char* message;
	int c, i;
	gint value;
	char* timeout;
	poptContext optCon;
	OPTIONS *my_options;
	char* win_title;
	GString *label;
	char *tmp;
	
	struct poptOption comman_line[] = {
		{ "buttons", 'b', POPT_ARG_STRING || POPT_ARGFLAG_ONEDASH, &buttons, 'b', 
				N_("Coma separated buttons list"), "label1[:val1],label2[:val2],..." },
		{ "default", 'd', POPT_ARG_STRING || POPT_ARGFLAG_ONEDASH, &default_label, 'd', 
				N_("Defines the default labelled button to be selected when [enter] is pressed"), "label" },
		{ "file", 'f', POPT_ARG_STRING || POPT_ARGFLAG_ONEDASH, &filename, 'f', 
 				N_("File to display. A filename of '-' reads from the standart input"), "filename" },
 		{ "print", 'p', POPT_ARGFLAG_ONEDASH, NULL, 'p',
 				N_("Prints the selected label on the standard output")},
 		{ "center", 'c', POPT_ARGFLAG_ONEDASH, NULL, 'c',
 				N_("Pop up the window at the center of the screen")},
 		{ "nearmouse", 'n', POPT_ARGFLAG_ONEDASH, NULL, 'n',
 				N_("Pop up the window near the mouse cursor")},
		{ "geometry", 'g', POPT_ARG_STRING || POPT_ARGFLAG_ONEDASH, &geometry, 'g', 
 				N_("Geometry. See man X for more info.") },
		{ "timeout", 't', POPT_ARG_STRING || POPT_ARGFLAG_ONEDASH, &timeout, 't', 
 				N_("Exits after t seconds if the user has not clicked any button yet.") },
		{ "wintitle", 'w', POPT_ARG_STRING || POPT_ARGFLAG_ONEDASH, &win_title, 'w', 
 				N_("Defines the Window Title."), "win_title"},
 		{ "version", 'v', POPT_ARGFLAG_ONEDASH, NULL, 'v',
 				N_("Prints the version of afxmessage")},
		POPT_AUTOHELP
		{ NULL, 0, 0, NULL, 0 }
	};

	my_options = (OPTIONS*)malloc(sizeof(OPTIONS));
	my_options->message = NULL;
	my_options->value = NULL;
	my_options->label = NULL;
	my_options->number = NULL;
	my_options->filename = NULL;
	my_options->print = FALSE;
	my_options->default_label = NULL;
	my_options->clicked_button = 0;
	my_options->win_position = GTK_WIN_POS_NONE;
	my_options->win_title = NULL;
	my_options->geometry = NULL;
	my_options->timeout = 0;

	
	optCon = poptGetContext(NULL, argc, argv, comman_line, 0);
	poptSetOtherOptionHelp(optCon, _("[OPTIONS]* message"));

	if (argc < 2) {
 		poptPrintUsage(optCon, stderr, 0);
		exit(1);
	}

	while ((c = poptGetNextOpt(optCon)) >= 0) {
		switch (c) {
			case 'b':
				parse_buttons(g_string_new(buttons), my_options);
				break;
			case 'd':
				my_options->default_label = g_string_new(default_label);
				break;
			case 'f':
				my_options->filename = g_string_new(filename);
				
				break;
			case 'p':
				my_options->print = TRUE;
				break;
			case 'c':
				my_options->win_position = GTK_WIN_POS_CENTER;
				break;
			case 'n':
				my_options->win_position = GTK_WIN_POS_MOUSE;
				break;
			case 'g':
				my_options->geometry = g_string_new(geometry);
				break;
			case 'w':
				my_options->win_title = g_string_new(win_title);
				break;
			case 't':
				my_options->timeout = atoi(timeout)*1000;
				break;
			case 'v' : 
				tmp = (char*)malloc(sizeof(char)*MAX_STRING_LENGTH);
				snprintf(tmp, MAX_STRING_LENGTH, 
					N_("afxmessage\nVersion %s\nLicence : GPL v2"), VERSION);
				my_options->message = g_string_new(tmp);
				free(tmp);
		}
	}
	message = poptGetArg(optCon);
	if(((message == NULL) || !(poptPeekArg(optCon) == NULL))
			&& (my_options->filename == NULL)
			&& (my_options->message == NULL))
		usage(optCon, 1, _("Specify a message to display\n"));

	if (c < -1) {
	/* an error occurred during option processing */
	fprintf(stderr, "%s: %s\n",
		poptBadOption(optCon, POPT_BADOPTION_NOALIAS),
		poptStrerror(c));
		return(NULL);
	}
	
	if(my_options->message == NULL) {
	
		if(my_options->filename == NULL) {
			my_options->message = g_string_new(message);
		} else {
			if(strcmp((my_options->filename)->str,"-")) {
				my_options->message = read_file(my_options->filename);		
			} else {
				my_options->message = read_pipe();
			}
		}
	}
    	    if(!my_options->label) {
    		    my_options->value = g_array_new(TRUE, TRUE, sizeof(gint));
    		    my_options->label = g_ptr_array_new();
    		    my_options->number = g_ptr_array_new();
    		    g_ptr_array_add(my_options->label, g_string_new(_("Okay")));
    		    g_ptr_array_add(my_options->number, g_string_new("0"));
    		    value = 0;
    		    g_array_append_val(my_options->value, value);

    	    }

    	    if(my_options->default_label) {
    		    i = 0;
    		    while(i < (my_options->number)->len) {
    			    label = g_ptr_array_index(my_options->label,i);
    			    if(strcmp(label->str, (my_options->default_label)->str) == 0) {
    				    my_options->clicked_button = i;
    				    i = (my_options->number)->len;
    			    }
    			    i++;
    		    }
    	    }
	return(my_options);
	
}
